@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 2007-2024. All Rights Reserved.
+%% Copyright Ericsson AB 2007-2025. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -55,7 +55,7 @@
 	 is_higher/2, supported_protocol_versions/0, sufficient_crypto_support/1,
 	 is_acceptable_version/1, is_acceptable_version/2, hello_version/1]).
 
--export_type([tls_version/0, tls_atom_version/0]).
+-export_type([tls_version/0, tls_atom_version/0, tls_max_frag_len/0]).
 
 -type tls_version()       :: ssl_record:ssl_version().
 -type tls_atom_version()  :: sslv3 | tlsv1 | 'tlsv1.1' | 'tlsv1.2' | 'tlsv1.3'.
@@ -459,7 +459,7 @@ initial_connection_state(ConnectionEnd, MaxEarlyDataSize) ->
       mac_secret  => undefined,
       early_data => #{pending_early_data_size => MaxEarlyDataSize,
                       trial_decryption => false,
-                      early_data_expected => false
+                      early_data_accepted => false
                      },
       reneg => #{secure_renegotiation => undefined,
                  client_verify_data => undefined,
